angular.module('app.controllers', ['fr.api'])
	.controller('StagesCtrl', ["$scope", "api", function ($scope, api) {
		api.getCompetitions().then(function(result) {
			$scope.stages = result;
		});
	}])
	.controller('MatchesCtrl', ["$scope", "api", "stage", function ($scope, api, stage) {
		$scope.stage = stage;
		
		$scope.moreDataCanBeLoaded = true;
		$scope.matches = [];
		var currentPage = 0;
		var lastPageShown = 0;
		var pagesPendingRendering = [];

		$scope.loadMore = function (clearCache) {
			var pageToLoad = ++currentPage;
			// console.debug("clearCache: " + clearCache);
			return api.getMatches(stage.tree, pageToLoad, clearCache).then(function (result) {
				// Sometimes multiple ajax requests are initiated at once to load multiple pages (e.g. - page 1 and 2 are loaded simultaneously).
				// This can result in page 2 being loaded before page 1. We need to ensure to always show results in the right order.
				// To do that we keep all pages in "cache" and only display them once all the previous pages have loaded successfully.
				pagesPendingRendering[pageToLoad] = result;
				
				$scope.$evalAsync(function () {
					// console.debug("page loaded: " + pageToLoad + ", last page shown: " + lastPageShown);

					if (lastPageShown === pageToLoad - 1) {
						for (var i = pageToLoad; angular.isDefined(pagesPendingRendering[i]); i++) {

							var matches = pagesPendingRendering[i].data;
							$scope.matches = $scope.matches.concat(matches);

							if (matches.length === 0) {
								$scope.moreDataCanBeLoaded = false;
							}

							delete pagesPendingRendering[i];
							lastPageShown = i;
						}

						$scope.$broadcast('scroll.infiniteScrollComplete');
					}
				});
			});
		};
		
		$scope.doRefresh = function() {
			$scope.moreDataCanBeLoaded = true;
			$scope.matches = [];
			currentPage = 0;
			lastPageShown = 0;
			pagesPendingRendering = [];

			$scope.loadMore(true).then(function() {
				$scope.$broadcast('scroll.refreshComplete');
			});
		};

		$scope.loadMore();

		api.getTables(stage.tree).then(function (result) {
			$scope.$evalAsync(function() {
				$scope.hasTable = !!result;
			});
		});
	}])
	.controller('MatchCtrl', ["$scope", "$sce", "api", "matchId", function ($scope, $sce, api, matchId) {
		function wrapVideo(content) {
			return content;
		}

		function getParameterByName(name, queryString) {
			name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
			var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
				results = regex.exec(queryString);
			return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
		}

		var getLocation = function (href) {
			var l = document.createElement("a");
			l.href = href;
			return l;
		};

		function getPlayer(url) {
			var location = getLocation(url), html;

			if (location.hostname.indexOf("playwire") >= 0) {
				var videoId = getParameterByName("videoId", url);
				var publisherId = getParameterByName("publisherId", url),
					config = getParameterByName("config", url);

				var script = "<script type='text/javascript' " +
					"charset='utf-8' " +
					"src='//cdn.playwire.com/bolt/js/embed.min.js' " +
					"data-width='250' " +
					"data-height='150' " +
					"data-config='" + config + "' " +
					"data-publisher-id='" + publisherId + "' " +
					"data-video-id='" + videoId + "'></script>";

				html = wrapVideo(script);
			} else {
				var videoUrl = url;

				if (location.hostname.indexOf("dailymotion") >= 0) {
					videoUrl = "http://www.dailymotion.com/embed/video" + url.match(/\/([x][^_]*)/gi);
				} else if (location.hostname.indexOf("longtailvideo") >= 0) {
					videoUrl = getParameterByName("file", url);
				} else if (location.hostname.indexOf("youtube") >= 0) {
					var indexOfV = url.indexOf("/v/");

					// If the url wasn't converted to the embed format.
					if (indexOfV >= 0) {
						videoUrl = "http://www.youtube.com/embed/" + url.substr(indexOfV + 3);
					}
				}

				html = wrapVideo("<iframe height='100%' width='100%' src='" + videoUrl + "' frameborder='0'></iframe>");
			}

			return {
				html: html,
				host: location.hostname
			}
		};

		api.getMatch(matchId).then(function (match) {
			$scope.$evalAsync(function() {
				$scope.match = match;

				if (match.post != null && !match.isEpl) {
					var player = getPlayer(match.post.media[0].payload);

					$scope.video = $sce.trustAsHtml(player.html);

					// Playwire doesn't open in full screen on android.
					$scope.fixFullScreen = player.host.indexOf("playwire" >= 0) && ionic.Platform.isAndroid();
				}
			});

			api.getTables(match.seasonStage).then(function (result) {
				$scope.$evalAsync(function () {
					$scope.hasTable = !!result;
				});
			});
		});
	}]);

angular.module("app", [
		"ionic",
		"ngCordova",
		"ionic.service.core",
		"ionic.service.push",
		"ionic.service.deploy",
		"app.controllers",
		"http-interceptor",
		"fr.api"
	])
	.config([
		"$ionicAppProvider", function($ionicAppProvider) {
			// Identify app
			$ionicAppProvider.identify({
				// The App ID (from apps.ionic.io) for the server
				app_id: "cfd1899c",
				// The public API key all services will use for this app
				api_key: "2f69405f2b0310f13556e89057e7a02aebf4037884c69701"
				// The GCM project ID (project number) from your Google Developer Console (uncomment if used)
				// gcm_id: 'GCM_ID'
			});
		}
	])
	.directive("myTextFit", function () {
		function calculateFontSize(string, clientWidth, defaultFontSize) {
			var defaultFontSizeAsNumber = parseInt(defaultFontSize, 10);
			var result = defaultFontSizeAsNumber;
			
			var fontSizeInPixels = Math.round(clientWidth / string.length * 2);

			if (!isNaN(fontSizeInPixels) && fontSizeInPixels < defaultFontSizeAsNumber) {
				result = fontSizeInPixels;
			}

			return result + "px";
		}

		return {
			link: function(scope, element) {
				var changeCount = 0;

				var removeWatch = scope.$watch(function () {
					return element[0].clientWidth;
				}, function (newValue) {
					changeCount += 1;

					// Normally it takes around 3 changes until layout stabilizes and the correct element width can be detected.
					if (changeCount < 3) { return; }

					// Make default font size a little larger by multiplying by 1.2.
					var defaultFontSize = document.defaultView.getComputedStyle(element[0]).getPropertyValue("font-size") * 1.2;

					var size = calculateFontSize(element.text(), newValue, defaultFontSize);
					element.css("font-size", size);

					// The font size shouldn't change again, so we simply remove the watch.
					removeWatch();
					return;
				});
			}
		};
	})
	.directive("myFullscreenVideo", function() {
		function getWindowSize() {
			var w = window,
				d = document,
				e = d.documentElement,
				g = d.getElementsByTagName('body')[0],
				height = w.innerHeight || e.clientHeight || g.clientHeight,
				width = w.innerWidth || e.clientWidth || g.clientWidth;

			return {
				w: width,
				h: height
			};
		}

		var suggestedVideoSize;

		function resize() {
			var windowSize = getWindowSize();
			
			// Store suggested video size in a global variable.
			suggestedVideoSize = suggestedVideoSize || {};
			suggestedVideoSize.height = (windowSize.h - 50);
			suggestedVideoSize.width = windowSize.w;
		}

		resize();
		
		var $window = angular.element(window);
		$window.on("resize", resize);

		function resizeVideo(el) {
			el.style.width = suggestedVideoSize.width + "px";
			el.style.height = suggestedVideoSize.height + "px";
		}

		return {
			link: function (scope, element) {
				var resizeElement = function () {
					scope.$evalAsync(function() {
						resizeVideo(element[0]);
						element.html(element.html());
					});
				};

				resizeElement();

				$window.on("resize", resizeElement);

				scope.$on('$destroy', function () {
					$window.off('resize', resizeElement);
				});
			}
		};
	})
	.directive('menuCloseForce', ['$ionicHistory', function ($ionicHistory) {
		return {
			restrict: 'AC',
			link: function ($scope, $element) {
				$element.bind('click', function () {
					var sideMenuCtrl = $element.inheritedData('$ionSideMenusController');
					if (sideMenuCtrl) {
						$ionicHistory.nextViewOptions({
							historyRoot: true,
							disableAnimate: true,
							expire: 300
						});
						
						sideMenuCtrl.close();
					}
				});
			}
		};
	}])
	.run(["$rootScope", "$ionicDeploy", "$ionicPlatform", "$cordovaStatusbar", "$ionicLoading", function ($rootScope, $ionicDeploy, $ionicPlatform, $cordovaStatusbar, $ionicLoading) {
		$ionicPlatform.ready(function() {

			// Hide the accessory bar by default
			if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
				cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
			}

			// Color the iOS status bar text to white
			if (window.StatusBar) {
				$cordovaStatusbar.overlaysWebView(true);
				$cordovaStatusbar.style(1); //Light
			}

			cordova.getAppVersion(function (version) {
				$rootScope.version = version;
			});

			$rootScope.updateOptions = {
				// Check for update every 2 minutes.
				interval: 2 * 60 * 1000
			};

			// Watch Ionic Deploy service for new code
			$ionicDeploy.watch($rootScope.updateOptions).then(function() {}, function() {}, function(hasUpdate) {
			    $rootScope.lastChecked = new Date();
			    
				if (hasUpdate) {
					$ionicDeploy.download().then(function () {
						// Extract the updates
						$ionicDeploy.extract().then(function () {
							// Load the updated version
							$ionicDeploy.load();

							// Clear progress.
							delete $rootScope.extractionProgress;
							delete $rootScope.downloadProgress;
						}, function (error) {
							// Error extracting
							delete $rootScope.extractionProgress;
						}, function (progress) {
							$rootScope.extractionProgress = progress;
						});
					}, function (error) {
						// Error downloading.
						delete $rootScope.downloadProgress;
					}, function (progress) {
						$rootScope.downloadProgress = progress;
					});
				}
			});

			//$rootScope.checkForUpdates = function () {
			//    console.log('Ionic Deploy: Checking for updates');
			//    $ionicDeploy.check().then(function (hasUpdate) {
			//        $ionicPopup.alert({
			//            title: "Update check",
			//            template: hasUpdate ? "New update is available." : "App is already up to date."
			//        });
			//        $rootScope.hasUpdate = hasUpdate;
			//    }, function (err) {
			//	    $ionicPopup.alert({
			//		    title: "Error",
			//		    template: JSON.stringify(err)
			//		});
			//    });
			//};
		});
		 
		var lastStateChangeOn = new Date();
		$rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
			var now = new Date();
			var diff = now.getTime() - lastStateChangeOn.getTime();
			
			// IOS has a bug causing "ghost" clicks. This causes double navigation in some cases. To avoid
			// double navigation we will ignore any duplicate state changes happening within 500ms window.
			if (diff < 500 && fromState.name !== "") {
				// Cancel state change.
				event.preventDefault();
				return;
			}
			
			lastStateChangeOn = now;

			$ionicLoading.show({
				template: 'Loading...',

				// Delay slightly so that for states that resolve immediately we won't show the loader.
				delay: 300
			});
		});

		$rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
			$ionicLoading.hide();
		});
	}])
	.config(["$stateProvider", "$urlRouterProvider", function ($stateProvider, $urlRouterProvider) {
		$stateProvider
			.state("home", {
				url: "/",
				templateUrl: "templates/matches.html",
				controller: "MatchesCtrl",
				resolve: {
					stage: function () {
						return {
							tree: "",
							name: "Latest games"
						};
					}
				}
			})
			.state("stage", {
				url: "/stage/:id",
				templateUrl: "templates/matches.html",
				controller: "MatchesCtrl",
				resolve: {
					stage: ["$stateParams", "api", function ($stateParams, api) {
						if ($stateParams.id === "") {
							return api.getCompetition({ tree: "" });
						}

						var tree = $stateParams.id.length > 0 ? $stateParams.id : null;
						return api.getCompetition({tree:tree});
					}]
				}
			})
			.state("stage-latest-season", {
				url: "/stage/latest-season/:id",
				templateUrl: "templates/matches.html",
				controller: "MatchesCtrl",
				resolve: {
					stage: ["$stateParams", "api", function ($stateParams, api) {
						if ($stateParams.id === "") {
							return api.getCompetition({ tree: "" });
						}

						var tree = $stateParams.id.length > 0 ? $stateParams.id : null;
						return api.getLatestSeason(tree);
					}]
				}
			})
			.state("match", {
				url: "/match/:id",
				templateUrl: "templates/match.html",
				controller: "MatchCtrl",
				resolve: {
					matchId: function ($stateParams) {
						return $stateParams.id;
					}
				}
			})
			.state("table", {
				url: "/table/:tree",
				templateUrl: "templates/table.html",
				controller: ["$scope", "api", "tree", "$q", function ($scope, api, tree, $q) {
					function load(clearCache) {
						var competitionStageId = tree.indexOf(",") > -1 ? tree.substr(0, tree.indexOf(",")) : tree;

						var promises = {
							tables: api.getTables(tree, !!clearCache),
							stage: api.getCompetition({ tree: competitionStageId })
						};

						return $q.all(promises).then(function(results) {
							$scope.$evalAsync(function() {
								$scope.tables = results.tables;
								$scope.stage = results.stage;
							});
						});
					}

					load(false);

					$scope.doRefresh = function () {
						load(true).then(function() {
							$scope.$broadcast('scroll.refreshComplete');
						});
					};
				}],
				resolve: {
					tree: function ($stateParams) {
						return $stateParams.tree;
					}
				}
			});

		$urlRouterProvider.otherwise("/");
	}]);
angular.module("fr.api", [])
	.constant("moment", moment)
	.service("api", ["$http", "$q", "$filter", "moment", function ($http, $q, $filter, moment) {
		var config = {
			api: "http://admin.footyroom.com/api/v2",
			content: "http://footyroom.com/mobile-app"
		};

		var cache = {},
			categoryRoot = null,
			self = this,
			matches = [];

		function clearCacheInGroup(cacheGroup) {
			if (typeof(cacheGroup) !== "string") {
				throw new Error("Must specify cacheGroup parameter.");
			}

			for (var property in cache) {
				if (cache.hasOwnProperty(property) && cache[property].group === cacheGroup) {
					delete cache[property];
				}
			}
		}

		function httpGet(uri, expiryInMilliseconds, clearCache, cacheGroup) {
			var deferred = $q.defer();
			var cacheItem = cache[uri];

			// If cache is available and is not expired.
			if (!clearCache && cacheItem != null && cacheItem.expiry > Date.now()) {
				deferred.resolve(cacheItem.value);
			} else {
				cache[uri] = {};
				// console.debug(uri);
				$http.get(uri, { cacheTimeout: 1000 * 10, retry: 10 }).success(function(result) {
					cache[uri] = {
						value: result,
						expiry: Date.now() + expiryInMilliseconds,
						group: cacheGroup
					};
					deferred.resolve(result);
				}).error(function() {
					deferred.reject();
				});
			}

			return deferred.promise;
		};

		function apiGet(uri, expiryInMilliseconds, clearCache, cacheGroup) {
			return httpGet(config.api + uri + "&api_key=f7d851bb79ac8a658e9c11d5e3c89f39", expiryInMilliseconds, clearCache, cacheGroup);
		}

		function getItem(getList, filter) {
			var deferred = $q.defer();

			getList().then(function(result) {
				var item = $filter('filter')(result, filter, true);

				if (item.length > 0) {
					deferred.resolve(item[0]);
				} else {
					deferred.resolve(null);
				}
			});

			return deferred.promise;
		}

		this.getCompetitions = function() {
			var deferred = $q.defer();
			var millisecondsInDay = 86400000;

			apiGet("/stages?hasPosts=1&type=competition", millisecondsInDay).then(function(stages) {
				if (stages[0].group === "latest") {
					deferred.resolve(stages);
					return;
				}

				stages = stages.filter(function(c) {
					// Hide Indian Super League (due to not working with them anymore).
					return c.id !== 2530;
				});

				categoryRoot = {
					id: 0,
					parent: null,
					name: "Latest games",
					group: null,
					sortRank: 0,
					tree: "",
					orderIndex: "/"
				};

				var groups = [];
				angular.forEach(stages, function(c) {
					var group = groups.find(function(g) {
						return g.name === c.group;
					});

					if (!group) {
						group = {
							id: c.id,
							name: c.group,
							parent: categoryRoot.id,
							level: "top",
							children: [],
							parentCategory: categoryRoot,
							sortRank: Number.MAX_VALUE
						};

						groups.push(group);
					}

					c.sortRank = c.sortRank || Number.MAX_VALUE;
					c.level = "node";
					c.parentCategory = group;
					c.children = [];

					group.children.push(c);
					group.sortRank = Math.min(group.sortRank, c.sortRank);
				});

				stages = stages.concat(groups);
				categoryRoot.children = groups;

				angular.forEach(stages, function(category) {
					var currentCategory = category;

					category.orderIndex = "";

					while (currentCategory != null && currentCategory !== categoryRoot) {
						category.orderIndex = pad(currentCategory.sortRank, 3) + "." + pad(currentCategory.id, 4) + "/" + category.orderIndex;

						currentCategory = currentCategory.parentCategory;
					}
				});

				// Prepend categoryRoot to the array.
				stages.unshift(categoryRoot);
				deferred.resolve(stages);
			});

			return deferred.promise;
		};

		this.getCompetition = function(filter) {
			return getItem(self.getCompetitions, filter);
		};

		this.getMatches = function(tree, page, clearCache) {
			var isLatestMatches = tree === "";

			if (clearCache) {
				// When clearing cache for matches, we need to clear cache for all pages for this particular tree.
				// This is necessary because we want to ensure that we don't clear cache just for the 1st, 2nd, etc page,
				// but rather for all pages. Otherwise we can end up in a situation where we've got latest version of the 1st page
				// but still looking at the old version of 2nd page, which can result in some data not being shown to the user.
				clearCacheInGroup(tree);
			}

			// For the "latest matches" we only want to show items that have posts, 
			// otherwise there will be lots of items without a video. For actual leagues
			// we want to show everything (i.e. - "hasPosts=0").
			var promise = apiGet("/matches?" + (isLatestMatches ? "has_post=1&" : "") + "status_type=finished&page=" + (page || 1) + "&stage_tree=" + tree, 1000 * 60 * 3, clearCache, tree);

			promise.then(function(result) {
				// Keep local cache of all matches, so that they can easily be found by `match_id` later.
				angular.forEach(result.data, function(match) {
					matches[match.match_id] = match;

					// Display all times in UTC, rather than in user's local time.
					var matchDate = moment(match.datetime.replace(" ", "T") + "Z").utc();
					match.date = matchDate.utc().format("LL");

					// Get stage tree for the season.
					match.seasonTree = match.stage_tree.substr(0, match.stage_tree.lastIndexOf(","));

					match.title = match.post != null ? match.post.post_title : (match.home_team.team_name + " " + match.home_score + " - " + match.away_score + " " + match.away_team.team_name);

					match.isEpl = match.stage_tree.indexOf("5,") === 0;

					// Season is always the second item in the tree. We need both competition
					// and season.
					match.seasonStage = match.stage_tree.split(",").slice(0, 2).join(",");
				});
			});

			return promise;
		};

		this.getMatch = function(matchId) {
			// Use promises (even though we resolve immediately), because in future we will probably
			// be retrieving by making an HTTP request.
			var deferred = $q.defer();
			deferred.resolve(matches[matchId]);
			return deferred.promise;
		};

		this.getLatestSeason = function(tree, includeAllStages) {
			var deferred = $q.defer();

			var competitionStageId = tree.indexOf(",") > -1 ? tree.substr(0, tree.indexOf(",")) : tree;

			apiGet("/stages?latestSeason=1&tree=" + competitionStageId, 1000 * 60 * 60 * 24).then(function(stages) {
				if (!includeAllStages) {
					var latestSeason = $filter("filter")(stages, { type: "season" }, true)[0];
					deferred.resolve(latestSeason);
				} else {
					deferred.resolve(stages);
				}
			});

			return deferred.promise;
		};

		this.getTables = function(tree, clearCache) {
			var deferred = $q.defer();

			if (typeof(tree) !== "string" || tree.length === 0) {
				deferred.resolve(null);
			} else {
				var promises = {
					stages: apiGet("/stages?tree=" + tree, 1000 * 60 * 60 * 24),
					tables: apiGet("/tables?stageTree=" + tree, 1000 * 60 * 10, clearCache)
				};

				$q.all(promises).then(function(results) {
					var tables = results.tables;
					var stages = results.stages;

					angular.forEach(tables, function(table) {
						var stage = $filter('filter')(stages, { tree: table.stageTree }, true)[0];
						table.stage = stage;
					});

					deferred.resolve(tables);
				});
			}

			return deferred.promise;
		};
	}]);
angular.module("http-interceptor", [])
	.config(['$httpProvider', function ($httpProvider) {
		$httpProvider.interceptors.push([
			'$location', '$injector', '$q', function ($location, $injector, $q) {
				// This interceptor does a couple of things:
				// * http retries

				return {
					'responseError': function (rejection) {
						if (rejection.status !== 404 && rejection.config.retry > 0) {
							// Decrement the number of retries.
							var newRequestConfig = angular.copy(rejection.config);
							newRequestConfig.retry -= 1;

							// console.debug("Retrying. Retries left: " + newRequestConfig.retry + ". " + rejection.config.url);

							// Retry.
							var $http = $injector.get('$http');
							return $http(newRequestConfig);
						}

						return $q.reject(rejection);
					}
				};
			}
		]);
	}
]);
if (!Array.prototype.find) {
	Array.prototype.find = function (predicate) {
		if (this === null) {
			throw new TypeError('Array.prototype.find called on null or undefined');
		}
		if (typeof predicate !== 'function') {
			throw new TypeError('predicate must be a function');
		}
		var list = Object(this);
		var length = list.length >>> 0;
		var thisArg = arguments[1];
		var value;

		for (var i = 0; i < length; i++) {
			value = list[i];
			if (predicate.call(thisArg, value, i, list)) {
				return value;
			}
		}
		return undefined;
	};
}

if (!Array.prototype.filter) {
	Array.prototype.filter = function (fun/*, thisArg*/) {
		'use strict';

		if (this === void 0 || this === null) {
			throw new TypeError();
		}

		var t = Object(this);
		var len = t.length >>> 0;
		if (typeof fun !== 'function') {
			throw new TypeError();
		}

		var res = [];
		var thisArg = arguments.length >= 2 ? arguments[1] : void 0;
		for (var i = 0; i < len; i++) {
			if (i in t) {
				var val = t[i];

				// NOTE: Technically this should Object.defineProperty at
				//       the next index, as push can be affected by
				//       properties on Object.prototype and Array.prototype.
				//       But that method's new, and collisions should be
				//       rare, so use the more-compatible alternative.
				if (fun.call(thisArg, val, i, t)) {
					res.push(val);
				}
			}
		}

		return res;
	};
}

function pad(n, width, z) {
	z = z || '0';
	n = n + '';
	return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
}