angular.module('app.controllers', ['fr.api'])
	.controller('StagesCtrl', ["$scope", "api", function ($scope, api) {
		api.getCompetitions().then(function(result) {
			$scope.stages = result;
		});
	}])
	.controller('MatchesCtrl', ["$scope", "api", "stage", function ($scope, api, stage) {
		$scope.stage = stage;
		
		$scope.moreDataCanBeLoaded = true;
		$scope.matches = [];
		var currentPage = 0;
		var lastPageShown = 0;
		var pagesPendingRendering = [];

		$scope.loadMore = function (clearCache) {
			var pageToLoad = ++currentPage;
			// console.debug("clearCache: " + clearCache);
			return api.getMatches(stage.tree, pageToLoad, clearCache).then(function (result) {
				// Sometimes multiple ajax requests are initiated at once to load multiple pages (e.g. - page 1 and 2 are loaded simultaneously).
				// This can result in page 2 being loaded before page 1. We need to ensure to always show results in the right order.
				// To do that we keep all pages in "cache" and only display them once all the previous pages have loaded successfully.
				pagesPendingRendering[pageToLoad] = result;
				
				$scope.$evalAsync(function () {
					// console.debug("page loaded: " + pageToLoad + ", last page shown: " + lastPageShown);

					if (lastPageShown === pageToLoad - 1) {
						for (var i = pageToLoad; angular.isDefined(pagesPendingRendering[i]); i++) {

							var matches = pagesPendingRendering[i].data;
							$scope.matches = $scope.matches.concat(matches);

							if (matches.length === 0) {
								$scope.moreDataCanBeLoaded = false;
							}

							delete pagesPendingRendering[i];
							lastPageShown = i;
						}

						$scope.$broadcast('scroll.infiniteScrollComplete');
					}
				});
			});
		};
		
		$scope.doRefresh = function() {
			$scope.moreDataCanBeLoaded = true;
			$scope.matches = [];
			currentPage = 0;
			lastPageShown = 0;
			pagesPendingRendering = [];

			$scope.loadMore(true).then(function() {
				$scope.$broadcast('scroll.refreshComplete');
			});
		};

		$scope.loadMore();

		api.getTables(stage.tree).then(function (result) {
			$scope.$evalAsync(function() {
				$scope.hasTable = !!result;
			});
		});
	}])
	.controller('MatchCtrl', ["$scope", "$sce", "api", "matchId", function ($scope, $sce, api, matchId) {
		function wrapVideo(content) {
			return content;
		}

		function getParameterByName(name, queryString) {
			name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
			var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
				results = regex.exec(queryString);
			return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
		}

		var getLocation = function (href) {
			var l = document.createElement("a");
			l.href = href;
			return l;
		};

		function getPlayer(url) {
			var location = getLocation(url), html;

			if (location.hostname.indexOf("playwire") >= 0) {
				var videoId = getParameterByName("videoId", url);
				var publisherId = getParameterByName("publisherId", url),
					config = getParameterByName("config", url);

				var script = "<script type='text/javascript' " +
					"charset='utf-8' " +
					"src='//cdn.playwire.com/bolt/js/embed.min.js' " +
					"data-width='250' " +
					"data-height='150' " +
					"data-config='" + config + "' " +
					"data-publisher-id='" + publisherId + "' " +
					"data-video-id='" + videoId + "'></script>";

				html = wrapVideo(script);
			} else {
				var videoUrl = url;

				if (location.hostname.indexOf("dailymotion") >= 0) {
					videoUrl = "http://www.dailymotion.com/embed/video" + url.match(/\/([x][^_]*)/gi);
				} else if (location.hostname.indexOf("longtailvideo") >= 0) {
					videoUrl = getParameterByName("file", url);
				} else if (location.hostname.indexOf("youtube") >= 0) {
					var indexOfV = url.indexOf("/v/");

					// If the url wasn't converted to the embed format.
					if (indexOfV >= 0) {
						videoUrl = "http://www.youtube.com/embed/" + url.substr(indexOfV + 3);
					}
				}

				html = wrapVideo("<iframe height='100%' width='100%' src='" + videoUrl + "' frameborder='0'></iframe>");
			}

			return {
				html: html,
				host: location.hostname
			}
		};

		api.getMatch(matchId).then(function (match) {
			$scope.$evalAsync(function() {
				$scope.match = match;

				if (match.post != null && !match.isEpl) {
					var player = getPlayer(match.post.media[0].payload);

					$scope.video = $sce.trustAsHtml(player.html);

					// Playwire doesn't open in full screen on android.
					$scope.fixFullScreen = player.host.indexOf("playwire" >= 0) && ionic.Platform.isAndroid();
				}
			});

			api.getTables(match.seasonStage).then(function (result) {
				$scope.$evalAsync(function () {
					$scope.hasTable = !!result;
				});
			});
		});
	}]);
