angular.module("app", [
		"ionic",
		"ngCordova",
		"ionic.service.core",
		"ionic.service.push",
		"ionic.service.deploy",
		"app.controllers",
		"http-interceptor",
		"fr.api"
	])
	.config([
		"$ionicAppProvider", function($ionicAppProvider) {
			// Identify app
			$ionicAppProvider.identify({
				// The App ID (from apps.ionic.io) for the server
				app_id: "cfd1899c",
				// The public API key all services will use for this app
				api_key: "2f69405f2b0310f13556e89057e7a02aebf4037884c69701"
				// The GCM project ID (project number) from your Google Developer Console (uncomment if used)
				// gcm_id: 'GCM_ID'
			});
		}
	])
	.directive("myTextFit", function () {
		function calculateFontSize(string, clientWidth, defaultFontSize) {
			var defaultFontSizeAsNumber = parseInt(defaultFontSize, 10);
			var result = defaultFontSizeAsNumber;
			
			var fontSizeInPixels = Math.round(clientWidth / string.length * 2);

			if (!isNaN(fontSizeInPixels) && fontSizeInPixels < defaultFontSizeAsNumber) {
				result = fontSizeInPixels;
			}

			return result + "px";
		}

		return {
			link: function(scope, element) {
				var changeCount = 0;

				var removeWatch = scope.$watch(function () {
					return element[0].clientWidth;
				}, function (newValue) {
					changeCount += 1;

					// Normally it takes around 3 changes until layout stabilizes and the correct element width can be detected.
					if (changeCount < 3) { return; }

					// Make default font size a little larger by multiplying by 1.2.
					var defaultFontSize = document.defaultView.getComputedStyle(element[0]).getPropertyValue("font-size") * 1.2;

					var size = calculateFontSize(element.text(), newValue, defaultFontSize);
					element.css("font-size", size);

					// The font size shouldn't change again, so we simply remove the watch.
					removeWatch();
					return;
				});
			}
		};
	})
	.directive("myFullscreenVideo", function() {
		function getWindowSize() {
			var w = window,
				d = document,
				e = d.documentElement,
				g = d.getElementsByTagName('body')[0],
				height = w.innerHeight || e.clientHeight || g.clientHeight,
				width = w.innerWidth || e.clientWidth || g.clientWidth;

			return {
				w: width,
				h: height
			};
		}

		var suggestedVideoSize;

		function resize() {
			var windowSize = getWindowSize();
			
			// Store suggested video size in a global variable.
			suggestedVideoSize = suggestedVideoSize || {};
			suggestedVideoSize.height = (windowSize.h - 50);
			suggestedVideoSize.width = windowSize.w;
		}

		resize();
		
		var $window = angular.element(window);
		$window.on("resize", resize);

		function resizeVideo(el) {
			el.style.width = suggestedVideoSize.width + "px";
			el.style.height = suggestedVideoSize.height + "px";
		}

		return {
			link: function (scope, element) {
				var resizeElement = function () {
					scope.$evalAsync(function() {
						resizeVideo(element[0]);
						element.html(element.html());
					});
				};

				resizeElement();

				$window.on("resize", resizeElement);

				scope.$on('$destroy', function () {
					$window.off('resize', resizeElement);
				});
			}
		};
	})
	.directive('menuCloseForce', ['$ionicHistory', function ($ionicHistory) {
		return {
			restrict: 'AC',
			link: function ($scope, $element) {
				$element.bind('click', function () {
					var sideMenuCtrl = $element.inheritedData('$ionSideMenusController');
					if (sideMenuCtrl) {
						$ionicHistory.nextViewOptions({
							historyRoot: true,
							disableAnimate: true,
							expire: 300
						});
						
						sideMenuCtrl.close();
					}
				});
			}
		};
	}])
	.run(["$rootScope", "$ionicDeploy", "$ionicPlatform", "$cordovaStatusbar", "$ionicLoading", function ($rootScope, $ionicDeploy, $ionicPlatform, $cordovaStatusbar, $ionicLoading) {
		$ionicPlatform.ready(function() {

			// Hide the accessory bar by default
			if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
				cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
			}

			// Color the iOS status bar text to white
			if (window.StatusBar) {
				$cordovaStatusbar.overlaysWebView(true);
				$cordovaStatusbar.style(1); //Light
			}

			cordova.getAppVersion(function (version) {
				$rootScope.version = version;
			});

			$rootScope.updateOptions = {
				// Check for update every 2 minutes.
				interval: 2 * 60 * 1000
			};

			// Watch Ionic Deploy service for new code
			$ionicDeploy.watch($rootScope.updateOptions).then(function() {}, function() {}, function(hasUpdate) {
			    $rootScope.lastChecked = new Date();
			    
				if (hasUpdate) {
					$ionicDeploy.download().then(function () {
						// Extract the updates
						$ionicDeploy.extract().then(function () {
							// Load the updated version
							$ionicDeploy.load();

							// Clear progress.
							delete $rootScope.extractionProgress;
							delete $rootScope.downloadProgress;
						}, function (error) {
							// Error extracting
							delete $rootScope.extractionProgress;
						}, function (progress) {
							$rootScope.extractionProgress = progress;
						});
					}, function (error) {
						// Error downloading.
						delete $rootScope.downloadProgress;
					}, function (progress) {
						$rootScope.downloadProgress = progress;
					});
				}
			});

			//$rootScope.checkForUpdates = function () {
			//    console.log('Ionic Deploy: Checking for updates');
			//    $ionicDeploy.check().then(function (hasUpdate) {
			//        $ionicPopup.alert({
			//            title: "Update check",
			//            template: hasUpdate ? "New update is available." : "App is already up to date."
			//        });
			//        $rootScope.hasUpdate = hasUpdate;
			//    }, function (err) {
			//	    $ionicPopup.alert({
			//		    title: "Error",
			//		    template: JSON.stringify(err)
			//		});
			//    });
			//};
		});
		 
		var lastStateChangeOn = new Date();
		$rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
			var now = new Date();
			var diff = now.getTime() - lastStateChangeOn.getTime();
			
			// IOS has a bug causing "ghost" clicks. This causes double navigation in some cases. To avoid
			// double navigation we will ignore any duplicate state changes happening within 500ms window.
			if (diff < 500 && fromState.name !== "") {
				// Cancel state change.
				event.preventDefault();
				return;
			}
			
			lastStateChangeOn = now;

			$ionicLoading.show({
				template: 'Loading...',

				// Delay slightly so that for states that resolve immediately we won't show the loader.
				delay: 300
			});
		});

		$rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
			$ionicLoading.hide();
		});
	}])
	.config(["$stateProvider", "$urlRouterProvider", function ($stateProvider, $urlRouterProvider) {
		$stateProvider
			.state("home", {
				url: "/",
				templateUrl: "templates/matches.html",
				controller: "MatchesCtrl",
				resolve: {
					stage: function () {
						return {
							tree: "",
							name: "Latest games"
						};
					}
				}
			})
			.state("stage", {
				url: "/stage/:id",
				templateUrl: "templates/matches.html",
				controller: "MatchesCtrl",
				resolve: {
					stage: ["$stateParams", "api", function ($stateParams, api) {
						if ($stateParams.id === "") {
							return api.getCompetition({ tree: "" });
						}

						var tree = $stateParams.id.length > 0 ? $stateParams.id : null;
						return api.getCompetition({tree:tree});
					}]
				}
			})
			.state("stage-latest-season", {
				url: "/stage/latest-season/:id",
				templateUrl: "templates/matches.html",
				controller: "MatchesCtrl",
				resolve: {
					stage: ["$stateParams", "api", function ($stateParams, api) {
						if ($stateParams.id === "") {
							return api.getCompetition({ tree: "" });
						}

						var tree = $stateParams.id.length > 0 ? $stateParams.id : null;
						return api.getLatestSeason(tree);
					}]
				}
			})
			.state("match", {
				url: "/match/:id",
				templateUrl: "templates/match.html",
				controller: "MatchCtrl",
				resolve: {
					matchId: function ($stateParams) {
						return $stateParams.id;
					}
				}
			})
			.state("table", {
				url: "/table/:tree",
				templateUrl: "templates/table.html",
				controller: ["$scope", "api", "tree", "$q", function ($scope, api, tree, $q) {
					function load(clearCache) {
						var competitionStageId = tree.indexOf(",") > -1 ? tree.substr(0, tree.indexOf(",")) : tree;

						var promises = {
							tables: api.getTables(tree, !!clearCache),
							stage: api.getCompetition({ tree: competitionStageId })
						};

						return $q.all(promises).then(function(results) {
							$scope.$evalAsync(function() {
								$scope.tables = results.tables;
								$scope.stage = results.stage;
							});
						});
					}

					load(false);

					$scope.doRefresh = function () {
						load(true).then(function() {
							$scope.$broadcast('scroll.refreshComplete');
						});
					};
				}],
				resolve: {
					tree: function ($stateParams) {
						return $stateParams.tree;
					}
				}
			});

		$urlRouterProvider.otherwise("/");
	}]);